(function() {
	'use strict';

	angular.module('ipconfigure.orchid.service').factory('IdleService', ['$document', '$log', '$timeout', IdleService]);

	function IdleService($document, $log, $timeout) {
		var _idleTickCount = 0,
			_watchers = [],
			_expectingAction = false;

		function _fireWatchers() {
			var i;

			$log.debug('IdleService idle for', _idleTickCount, 'seconds.', (_watchers[_idleTickCount]) ? _watchers[_idleTickCount].length : 0, 'listeners.');

			if (_watchers[_idleTickCount]) {
				// iterate through array and fire callbacks
				for (i = 0; i < _watchers[_idleTickCount].length; i += 1) {
					_watchers[_idleTickCount][i]();
				}

				_expectingAction = true;
			}
		}

		function _onDocumentActivity() {
			_idleTickCount = 0;

			// only fire this if someone is expecting it (meaning we've broadcast another message prior to this)
			if (_expectingAction === true) {
				_fireWatchers();

				_expectingAction = false;
			}
		}

		function _cancelMaker(seconds, callback) {
			return function() {
				var i;

				// remove the callback
				if (Array.isArray(_watchers[seconds]) === true) {
					for (i = 0; i < _watchers[seconds].length; i += 1) {
						if (_watchers[seconds][i] === callback) {
							_watchers[seconds].splice(i, 1);

							if (_watchers[seconds].length === 0) {
								delete _watchers[seconds];
							}

							return true;
						}
					}
				}

				return false;
			};
		}

		function _subscribe(seconds, callback) {
			// keep track of the callback
			if (Array.isArray(_watchers[seconds]) === true) {
				_watchers[seconds].push(callback);
			}
			else {
				_watchers[seconds] = [callback];
			}

			// return a cancel function
			return _cancelMaker(seconds, callback);
		}

		function _onUserIdleFor(seconds, callback) {
			return _subscribe(seconds, callback);
		}

		function _onUserAction(callback) {
			return _subscribe(0, callback);
		}

		function _init() {
			// monitor for mousemove so we know when to chill and when to wile out
			$document.bind('keydown mousedown mousemove mousewheel DOMMouseScroll click TouchStart', _onDocumentActivity);

			// ~infinite loop checking for inactivity
			(function loop() {
				_idleTickCount++;

				// fire event if a subscription matches the current idleTickCount
				_fireWatchers();

				$timeout(loop, 1000);
			}());
		}

		_init();

		return {
			onUserIdleFor: _onUserIdleFor,
			onUserAction: _onUserAction
		};
	}
}());
